@@ -1,93 +1,138 @@
 """Server for mypy daemon mode.
 
-Highly experimental!  Only supports UNIX-like systems.
+Only supports UNIX-like systems.
 
 This implements a daemon process which keeps useful state in memory
 to enable fine-grained incremental reprocessing of changes.
 """
 
-import gc
-import io
+import argparse
+import base64
 import json
 import os
-import shutil
-import socket
+import pickle
+import random
+import subprocess
 import sys
 import tempfile
 import time
 import traceback
 
-from typing import Any, Callable, Dict, List, Mapping, Optional, Sequence, Tuple
+from typing import AbstractSet, Any, Callable, Dict, List, Mapping, Optional, Sequence, Set, Tuple
 
 import mypy.build
 import mypy.errors
+import mypy.main
 from mypy.find_sources import create_source_list, InvalidSourceList
 from mypy.server.update import FineGrainedBuildManager
 from mypy.dmypy_util import STATUS_FILE, receive
-from mypy.gclogger import GcLogger
+from mypy.ipc import IPCServer, IPCException
 from mypy.fscache import FileSystemCache
 from mypy.fswatcher import FileSystemWatcher, FileData
+from mypy.modulefinder import BuildSource, compute_search_paths
 from mypy.options import Options
 from mypy.typestate import reset_global_state
 from mypy.version import __version__
 
 
-MEM_PROFILE = False  # If True, dump memory profile after initialization
+MYPY = False
+if MYPY:
+    from typing_extensions import Final
 
+MEM_PROFILE = False  # type: Final  # If True, dump memory profile after initialization
 
-def daemonize(func: Callable[[], None], log_file: Optional[str] = None) -> int:
-    """Arrange to call func() in a grandchild of the current process.
+if sys.platform == 'win32':
+    from subprocess import STARTUPINFO
 
-    Return 0 for success, exit status for failure, negative if
-    subprocess killed by signal.
-    """
-    # See https://stackoverflow.com/questions/473620/how-do-you-create-a-daemon-in-python
-    sys.stdout.flush()
-    sys.stderr.flush()
-    pid = os.fork()
-    if pid:
-        # Parent process: wait for child in case things go bad there.
-        npid, sts = os.waitpid(pid, 0)
-        sig = sts & 0xff
-        if sig:
-            print("Child killed by signal", sig)
-            return -sig
-        sts = sts >> 8
-        if sts:
-            print("Child exit status", sts)
-        return sts
-    # Child process: do a bunch of UNIX stuff and then fork a grandchild.
-    try:
-        os.setsid()  # Detach controlling terminal
-        os.umask(0o27)
-        devnull = os.open('/dev/null', os.O_RDWR)
-        os.dup2(devnull, 0)
-        os.dup2(devnull, 1)
-        os.dup2(devnull, 2)
-        os.close(devnull)
+    def daemonize(options: Options,
+                  timeout: Optional[int] = None,
+                  log_file: Optional[str] = None) -> int:
+        """Create the daemon process via "dmypy daemon" and pass options via command line
+
+        When creating the daemon grandchild, we create it in a new console, which is
+        started hidden. We cannot use DETACHED_PROCESS since it will cause console windows
+        to pop up when starting. See
+        https://github.com/python/cpython/pull/4150#issuecomment-340215696
+        for more on why we can't have nice things.
+
+        It also pickles the options to be unpickled by mypy.
+        """
+        command = [sys.executable, '-m', 'mypy.dmypy', 'daemon']
+        pickeled_options = pickle.dumps((options.snapshot(), timeout, log_file))
+        command.append('--options-data="{}"'.format(base64.b64encode(pickeled_options).decode()))
+        info = STARTUPINFO(dwFlags=0x1,  # STARTF_USESHOWWINDOW aka use wShowWindow's value
+                           wShowWindow=0,  # SW_HIDE aka make the window invisible
+                           )
+        try:
+            subprocess.Popen(command,
+                             creationflags=0x10,  # CREATE_NEW_CONSOLE
+                             startupinfo=info)
+            return 0
+        except subprocess.CalledProcessError as e:
+            return e.returncode
+
+else:
+    def _daemonize_cb(func: Callable[[], None], log_file: Optional[str] = None) -> int:
+        """Arrange to call func() in a grandchild of the current process.
+
+        Return 0 for success, exit status for failure, negative if
+        subprocess killed by signal.
+        """
+        # See https://stackoverflow.com/questions/473620/how-do-you-create-a-daemon-in-python
+        sys.stdout.flush()
+        sys.stderr.flush()
         pid = os.fork()
         if pid:
-            # Child is done, exit to parent.
-            os._exit(0)
-        # Grandchild: run the server.
-        if log_file:
-            sys.stdout = sys.stderr = open(log_file, 'a', buffering=1)
-            fd = sys.stdout.fileno()
-            os.dup2(fd, 2)
-            os.dup2(fd, 1)
-        func()
-    finally:
-        # Make sure we never get back into the caller.
-        os._exit(1)
+            # Parent process: wait for child in case things go bad there.
+            npid, sts = os.waitpid(pid, 0)
+            sig = sts & 0xff
+            if sig:
+                print("Child killed by signal", sig)
+                return -sig
+            sts = sts >> 8
+            if sts:
+                print("Child exit status", sts)
+            return sts
+        # Child process: do a bunch of UNIX stuff and then fork a grandchild.
+        try:
+            os.setsid()  # Detach controlling terminal
+            os.umask(0o27)
+            devnull = os.open('/dev/null', os.O_RDWR)
+            os.dup2(devnull, 0)
+            os.dup2(devnull, 1)
+            os.dup2(devnull, 2)
+            os.close(devnull)
+            pid = os.fork()
+            if pid:
+                # Child is done, exit to parent.
+                os._exit(0)
+            # Grandchild: run the server.
+            if log_file:
+                sys.stdout = sys.stderr = open(log_file, 'a', buffering=1)
+                fd = sys.stdout.fileno()
+                os.dup2(fd, 2)
+                os.dup2(fd, 1)
+            func()
+        finally:
+            # Make sure we never get back into the caller.
+            os._exit(1)
 
+    def daemonize(options: Options,
+                  timeout: Optional[int] = None,
+                  log_file: Optional[str] = None) -> int:
+        """Run the mypy daemon in a grandchild of the current process
+
+        Return 0 for success, exit status for failure, negative if
+        subprocess killed by signal.
+        """
+        return _daemonize_cb(Server(options, timeout).serve, log_file)
 
 # Server code.
 
-SOCKET_NAME = 'dmypy.sock'
+CONNECTION_NAME = 'dmypy.sock'  # type: Final
 
 
 def process_start_options(flags: List[str], allow_sources: bool) -> Options:
-    import mypy.main
     sources, options = mypy.main.process_options(['-i'] + flags,
                                                  require_targets=False,
                                                  server_options=True)
@@ -109,6 +154,11 @@ def process_start_options(flags: List[str], allow_sources: bool) -> Options:
     return options
 
 
+ModulePathPair = Tuple[str, str]
+ModulePathPairs = List[ModulePathPair]
+ChangesAndRemovals = Tuple[ModulePathPairs, ModulePathPairs]
+
+
 class Server:
 
     # NOTE: the instance is constructed in the parent process but
@@ -143,26 +193,15 @@ class Server:
 
     def serve(self) -> None:
         """Serve requests, synchronously (no thread or fork)."""
+        command = None
         try:
-            sock = self.create_listening_socket()
-            if self.timeout is not None:
-                sock.settimeout(self.timeout)
-            try:
-                with open(STATUS_FILE, 'w') as f:
-                    json.dump({'pid': os.getpid(), 'sockname': sock.getsockname()}, f)
-                    f.write('\n')  # I like my JSON with trailing newline
-                while True:
-                    try:
-                        conn, addr = sock.accept()
-                    except socket.timeout:
-                        print("Exiting due to inactivity.")
-                        reset_global_state()
-                        sys.exit(0)
-                    try:
-                        data = receive(conn)
-                    except OSError as err:
-                        conn.close()  # Maybe the client hung up
-                        continue
+            server = IPCServer(CONNECTION_NAME, self.timeout)
+            with open(STATUS_FILE, 'w') as f:
+                json.dump({'pid': os.getpid(), 'connection_name': server.connection_name}, f)
+                f.write('\n')  # I like my JSON with a trailing newline
+            while True:
+                with server:
+                    data = receive(server)
                     resp = {}  # type: Dict[str, Any]
                     if 'command' not in data:
                         resp = {'error': "No command found in request"}
@@ -172,39 +211,36 @@ class Server:
                             resp = {'error': "Command is not a string"}
                         else:
                             command = data.pop('command')
-                        try:
-                            resp = self.run_command(command, data)
-                        except Exception:
-                            # If we are crashing, report the crash to the client
-                            tb = traceback.format_exception(*sys.exc_info())  # type: ignore
-                            resp = {'error': "Daemon crashed!\n" + "".join(tb)}
-                            conn.sendall(json.dumps(resp).encode('utf8'))
-                            raise
+                            try:
+                                resp = self.run_command(command, data)
+                            except Exception:
+                                # If we are crashing, report the crash to the client
+                                tb = traceback.format_exception(*sys.exc_info())
+                                resp = {'error': "Daemon crashed!\n" + "".join(tb)}
+                                server.write(json.dumps(resp).encode('utf8'))
+                                raise
                     try:
-                        conn.sendall(json.dumps(resp).encode('utf8'))
-                    except OSError as err:
+                        server.write(json.dumps(resp).encode('utf8'))
+                    except OSError:
                         pass  # Maybe the client hung up
-                    conn.close()
                     if command == 'stop':
-                        sock.close()
                         reset_global_state()
                         sys.exit(0)
-            finally:
-                os.unlink(STATUS_FILE)
         finally:
-            shutil.rmtree(self.sock_directory)
+            # If the final command is something other than a clean
+            # stop, remove the status file. (We can't just
+            # simplify the logic and always remove the file, since
+            # that could cause us to remove a future server's
+            # status file.)
+            if command != 'stop':
+                os.unlink(STATUS_FILE)
+            try:
+                server.cleanup()  # try to remove the socket dir on Linux
+            except OSError:
+                pass
             exc_info = sys.exc_info()
             if exc_info[0] and exc_info[0] is not SystemExit:
-                traceback.print_exception(*exc_info)  # type: ignore
-
-    def create_listening_socket(self) -> socket.socket:
-        """Create the socket and set it up for listening."""
-        self.sock_directory = tempfile.mkdtemp()
-        sockname = os.path.join(self.sock_directory, SOCKET_NAME)
-        sock = socket.socket(socket.AF_UNIX)
-        sock.bind(sockname)
-        sock.listen(1)
-        return sock
+                traceback.print_exception(*exc_info)
 
     def run_command(self, command: str, data: Mapping[str, object]) -> Dict[str, object]:
         """Run a specific command from the registry."""
@@ -225,14 +261,17 @@ class Server:
 
     def cmd_stop(self) -> Dict[str, object]:
         """Stop daemon."""
+        # We need to remove the status file *before* we complete the
+        # RPC. Otherwise a race condition exists where a subsequent
+        # command can see a status file from a dying server and think
+        # it is a live one.
+        os.unlink(STATUS_FILE)
         return {}
 
-    last_sources = None  # type: List[mypy.build.BuildSource]
-
     def cmd_run(self, version: str, args: Sequence[str]) -> Dict[str, object]:
         """Check a list of files, triggering a restart if needed."""
         try:
-            self.last_sources, options = mypy.main.process_options(
+            sources, options = mypy.main.process_options(
                 ['-i'] + list(args),
                 require_targets=True,
                 server_options=True,
@@ -242,25 +281,55 @@ class Server:
                 return {'restart': 'configuration changed'}
             if __version__ != version:
                 return {'restart': 'mypy version changed'}
+            if self.fine_grained_manager:
+                manager = self.fine_grained_manager.manager
+                start_plugins_snapshot = manager.plugins_snapshot
+                _, current_plugins_snapshot = mypy.build.load_plugins(options, manager.errors)
+                if current_plugins_snapshot != start_plugins_snapshot:
+                    return {'restart': 'plugins changed'}
         except InvalidSourceList as err:
             return {'out': '', 'err': str(err), 'status': 2}
-        return self.check(self.last_sources)
+        return self.check(sources)
 
     def cmd_check(self, files: Sequence[str]) -> Dict[str, object]:
         """Check a list of files."""
         try:
-            self.last_sources = create_source_list(files, self.options, self.fscache)
+            sources = create_source_list(files, self.options, self.fscache)
         except InvalidSourceList as err:
             return {'out': '', 'err': str(err), 'status': 2}
-        return self.check(self.last_sources)
+        return self.check(sources)
 
-    def cmd_recheck(self) -> Dict[str, object]:
-        """Check the same list of files we checked most recently."""
-        if not self.last_sources:
+    def cmd_recheck(self,
+                    remove: Optional[List[str]] = None,
+                    update: Optional[List[str]] = None) -> Dict[str, object]:
+        """Check the same list of files we checked most recently.
+
+        If remove/update is given, they modify the previous list;
+        if all are None, stat() is called for each file in the previous list.
+        """
+        t0 = time.time()
+        if not self.fine_grained_manager:
             return {'error': "Command 'recheck' is only valid after a 'check' command"}
-        return self.check(self.last_sources)
+        sources = self.previous_sources
+        if remove:
+            removals = set(remove)
+            sources = [s for s in sources if s.path and s.path not in removals]
+        if update:
+            known = {s.path for s in sources if s.path}
+            added = [p for p in update if p not in known]
+            try:
+                added_sources = create_source_list(added, self.options, self.fscache)
+            except InvalidSourceList as err:
+                return {'out': '', 'err': str(err), 'status': 2}
+            sources = sources + added_sources  # Make a copy!
+        t1 = time.time()
+        manager = self.fine_grained_manager.manager
+        manager.log("fine-grained increment: cmd_recheck: {:.3f}s".format(t1 - t0))
+        res = self.fine_grained_increment(sources, remove, update)
+        self.fscache.flush()
+        return res
 
-    def check(self, sources: List[mypy.build.BuildSource]) -> Dict[str, Any]:
+    def check(self, sources: List[BuildSource]) -> Dict[str, Any]:
         """Check using fine-grained incremental mode."""
         if not self.fine_grained_manager:
             res = self.initialize_fine_grained(sources)
@@ -269,7 +338,7 @@ class Server:
         self.fscache.flush()
         return res
 
-    def initialize_fine_grained(self, sources: List[mypy.build.BuildSource]) -> Dict[str, Any]:
+    def initialize_fine_grained(self, sources: List[BuildSource]) -> Dict[str, Any]:
         self.fswatcher = FileSystemWatcher(self.fscache)
         self.update_sources(sources)
         try:
@@ -322,36 +391,57 @@ class Server:
         status = 1 if messages else 0
         return {'out': ''.join(s + '\n' for s in messages), 'err': '', 'status': status}
 
-    def fine_grained_increment(self, sources: List[mypy.build.BuildSource]) -> Dict[str, Any]:
+    def fine_grained_increment(self,
+                               sources: List[BuildSource],
+                               remove: Optional[List[str]] = None,
+                               update: Optional[List[str]] = None,
+                               ) -> Dict[str, Any]:
         assert self.fine_grained_manager is not None
         manager = self.fine_grained_manager.manager
 
         t0 = time.time()
-        self.update_sources(sources)
-        changed, removed = self.find_changed(sources)
-        manager.lib_path = tuple(mypy.build.compute_lib_path(
-            sources, manager.options, manager.data_dir))
+        if remove is None and update is None:
+            # Use the fswatcher to determine which files were changed
+            # (updated or added) or removed.
+            self.update_sources(sources)
+            changed, removed = self.find_changed(sources)
+        else:
+            # Use the remove/update lists to update fswatcher.
+            # This avoids calling stat() for unchanged files.
+            changed, removed = self.update_changed(sources, remove or [], update or [])
+        manager.search_paths = compute_search_paths(sources, manager.options, manager.data_dir)
         t1 = time.time()
+        manager.log("fine-grained increment: find_changed: {:.3f}s".format(t1 - t0))
         messages = self.fine_grained_manager.update(changed, removed)
         t2 = time.time()
-        manager.log(
-            "fine-grained increment: find_changed: {:.3f}s, update: {:.3f}s".format(
-                t1 - t0, t2 - t1))
+        manager.log("fine-grained increment: update: {:.3f}s".format(t2 - t1))
         status = 1 if messages else 0
         self.previous_sources = sources
         return {'out': ''.join(s + '\n' for s in messages), 'err': '', 'status': status}
 
-    def update_sources(self, sources: List[mypy.build.BuildSource]) -> None:
+    def update_sources(self, sources: List[BuildSource]) -> None:
         paths = [source.path for source in sources if source.path is not None]
         self.fswatcher.add_watched_paths(paths)
 
-    def find_changed(self, sources: List[mypy.build.BuildSource]) -> Tuple[List[Tuple[str, str]],
-                                                                           List[Tuple[str, str]]]:
+    def update_changed(self,
+                       sources: List[BuildSource],
+                       remove: List[str],
+                       update: List[str],
+                       ) -> ChangesAndRemovals:
+
+        changed_paths = self.fswatcher.update_changed(remove, update)
+        return self._find_changed(sources, changed_paths)
+
+    def find_changed(self, sources: List[BuildSource]) -> ChangesAndRemovals:
         changed_paths = self.fswatcher.find_changed()
+        return self._find_changed(sources, changed_paths)
+
+    def _find_changed(self, sources: List[BuildSource],
+                      changed_paths: AbstractSet[str]) -> ChangesAndRemovals:
         # Find anything that has been added or modified
         changed = [(source.module, source.path)
                    for source in sources
-                   if source.path in changed_paths]
+                   if source.path and source.path in changed_paths]
 
         # Now find anything that has been removed from the build
         modules = {source.module for source in sources}
@@ -382,20 +472,11 @@ class Server:
 # Misc utilities.
 
 
-MiB = 2**20
+MiB = 2**20  # type: Final
 
 
 def get_meminfo() -> Dict[str, Any]:
-    # See https://stackoverflow.com/questions/938733/total-memory-used-by-python-process
-    import resource  # Since it doesn't exist on Windows.
     res = {}  # type: Dict[str, Any]
-    rusage = resource.getrusage(resource.RUSAGE_SELF)
-    if sys.platform == 'darwin':
-        factor = 1
-    else:
-        factor = 1024  # Linux
-    res['memory_maxrss_mib'] = rusage.ru_maxrss * factor / MiB
-    # If we can import psutil, use it for some extra data
     try:
         import psutil  # type: ignore  # It's not in typeshed yet
     except ImportError:
@@ -409,4 +490,17 @@ def get_meminfo() -> Dict[str, Any]:
         meminfo = process.memory_info()
         res['memory_rss_mib'] = meminfo.rss / MiB
         res['memory_vms_mib'] = meminfo.vms / MiB
+        if sys.platform == 'win32':
+            res['memory_maxrss_mib'] = meminfo.peak_wset / MiB
+        else:
+            # See https://stackoverflow.com/questions/938733/total-memory-used-by-python-process
+            import resource  # Since it doesn't exist on Windows.
+            rusage = resource.getrusage(resource.RUSAGE_SELF)
+            # mypyc doesn't like unreachable code, so trick mypy into thinking
+            # the branch is reachable
+            if sys.platform == 'darwin' or bool():
+                factor = 1
+            else:
+                factor = 1024  # Linux
+            res['memory_maxrss_mib'] = rusage.ru_maxrss * factor / MiB
     return res
