@@ -9,7 +9,7 @@ from mypy_extensions import trait
 
 from mypy.nodes import (
     Expression, StrExpr, IntExpr, UnaryExpr, Context, DictExpr, ClassDef,
-    TypeInfo, SymbolTableNode, MypyFile
+    TypeInfo, SymbolTableNode, MypyFile, CallExpr
 )
 from mypy.tvar_scope import TypeVarScope
 from mypy.types import (
@@ -25,6 +25,8 @@ import mypy.interpreted_plugin
 class TypeAnalyzerPluginInterface:
     """Interface for accessing semantic analyzer functionality in plugins."""
 
+    options = None  # type: Options
+
     @abstractmethod
     def fail(self, msg: str, ctx: Context) -> None:
         raise NotImplementedError
@@ -57,6 +59,7 @@ class CheckerPluginInterface:
     """Interface for accessing type checker functionality in plugins."""
 
     msg = None  # type: MessageBuilder
+    options = None  # type: Options
 
     @abstractmethod
     def named_generic_type(self, name: str, args: List[Type]) -> Instance:
@@ -69,6 +72,7 @@ class SemanticAnalyzerPluginInterface:
 
     modules = None  # type: Dict[str, MypyFile]
     options = None  # type: Options
+    cur_mod_id = None  # type: str
     msg = None  # type: MessageBuilder
 
     @abstractmethod
@@ -113,6 +117,19 @@ class SemanticAnalyzerPluginInterface:
                          suppress_errors: bool = False) -> Optional[SymbolTableNode]:
         raise NotImplementedError
 
+    @abstractmethod
+    def add_plugin_dependency(self, trigger: str, target: Optional[str] = None) -> None:
+        raise NotImplementedError
+
+    @abstractmethod
+    def add_symbol_table_node(self, name: str, stnode: SymbolTableNode) -> None:
+        """Add node to global symbol table (or to nearest class if there is one)."""
+        raise NotImplementedError
+
+    @abstractmethod
+    def qualified_name(self, n: str) -> str:
+        raise NotImplementedError
+
 
 # A context for a function hook that infers the return type of a function with
 # a special signature.
@@ -161,12 +178,21 @@ AttributeContext = NamedTuple(
 
 # A context for a class hook that modifies the class definition.
 ClassDefContext = NamedTuple(
-    'ClassDecoratorContext', [
+    'ClassDefContext', [
         ('cls', ClassDef),       # The class definition
         ('reason', Expression),  # The expression being applied (decorator, metaclass, base class)
         ('api', SemanticAnalyzerPluginInterface)
     ])
 
+# A context for dynamic class definitions like
+# Base = declarative_base()
+DynamicClassDefContext = NamedTuple(
+    'DynamicClassDefContext', [
+        ('call', CallExpr),      # The r.h.s. of dynamic class definition
+        ('name', str),  # The name this class is being assigned to
+        ('api', SemanticAnalyzerPluginInterface)
+    ])
+
 
 class Plugin:
     """Base class of all type checker plugins.
@@ -221,6 +247,10 @@ class Plugin:
                                      ) -> Optional[Callable[[ClassDefContext], None]]:
         return None
 
+    def get_dynamic_class_hook(self, fullname: str
+                               ) -> Optional[Callable[[DynamicClassDefContext], None]]:
+        return None
+
 
 T = TypeVar('T')
 
@@ -276,6 +306,10 @@ class WrapperPlugin(Plugin):
                                      ) -> Optional[Callable[[ClassDefContext], None]]:
         return self.plugin.get_customize_class_mro_hook(fullname)
 
+    def get_dynamic_class_hook(self, fullname: str
+                               ) -> Optional[Callable[[DynamicClassDefContext], None]]:
+        return self.plugin.get_dynamic_class_hook(fullname)
+
 
 class ChainedPlugin(Plugin):
     """A plugin that represents a sequence of chained plugins.
@@ -333,6 +367,10 @@ class ChainedPlugin(Plugin):
                                      ) -> Optional[Callable[[ClassDefContext], None]]:
         return self._find_hook(lambda plugin: plugin.get_customize_class_mro_hook(fullname))
 
+    def get_dynamic_class_hook(self, fullname: str
+                               ) -> Optional[Callable[[DynamicClassDefContext], None]]:
+        return self._find_hook(lambda plugin: plugin.get_dynamic_class_hook(fullname))
+
     def _find_hook(self, lookup: Callable[[Plugin], T]) -> Optional[T]:
         for plugin in self._plugins:
             hook = lookup(plugin)
@@ -346,24 +384,48 @@ class DefaultPlugin(Plugin):
 
     def get_function_hook(self, fullname: str
                           ) -> Optional[Callable[[FunctionContext], Type]]:
+        from mypy.plugins import ctypes
+
         if fullname == 'contextlib.contextmanager':
             return contextmanager_callback
         elif fullname == 'builtins.open' and self.python_version[0] == 3:
             return open_callback
+        elif fullname == 'ctypes.Array':
+            return ctypes.array_constructor_callback
         return None
 
     def get_method_signature_hook(self, fullname: str
                                   ) -> Optional[Callable[[MethodSigContext], CallableType]]:
+        from mypy.plugins import ctypes
+
         if fullname == 'typing.Mapping.get':
             return typed_dict_get_signature_callback
+        elif fullname == 'ctypes.Array.__setitem__':
+            return ctypes.array_setitem_callback
         return None
 
     def get_method_hook(self, fullname: str
                         ) -> Optional[Callable[[MethodContext], Type]]:
+        from mypy.plugins import ctypes
+
         if fullname == 'typing.Mapping.get':
             return typed_dict_get_callback
         elif fullname == 'builtins.int.__pow__':
             return int_pow_callback
+        elif fullname == 'ctypes.Array.__getitem__':
+            return ctypes.array_getitem_callback
+        elif fullname == 'ctypes.Array.__iter__':
+            return ctypes.array_iter_callback
+        return None
+
+    def get_attribute_hook(self, fullname: str
+                           ) -> Optional[Callable[[AttributeContext], Type]]:
+        from mypy.plugins import ctypes
+
+        if fullname == 'ctypes.Array.value':
+            return ctypes.array_value_callback
+        elif fullname == 'ctypes.Array.raw':
+            return ctypes.array_raw_callback
         return None
 
     def get_class_decorator_hook(self, fullname: str
