@@ -59,7 +59,7 @@ from mypy.nodes import (
 from mypy.types import (
     Type, TypeVisitor, UnboundType, AnyType, NoneTyp, UninhabitedType,
     ErasedType, DeletedType, Instance, TypeVarType, CallableType, TupleType, TypedDictType,
-    UnionType, Overloaded, PartialType, TypeType
+    UnionType, Overloaded, PartialType, TypeType, LiteralType,
 )
 from mypy.util import get_prefix
 
@@ -175,9 +175,14 @@ def snapshot_definition(node: Optional[SymbolNode],
             signature = snapshot_type(node.type)
         else:
             signature = snapshot_untyped_signature(node)
-        return ('Func', common, node.is_property, node.is_class, node.is_static, signature)
+        return ('Func', common,
+                node.is_property, node.is_final,
+                node.is_class, node.is_static,
+                signature)
     elif isinstance(node, Var):
-        return ('Var', common, snapshot_optional_type(node.type))
+        return ('Var', common,
+                snapshot_optional_type(node.type),
+                node.is_final)
     elif isinstance(node, Decorator):
         # Note that decorated methods are represented by Decorator instances in
         # a symbol table since we need to preserve information about the
@@ -310,6 +315,9 @@ class SnapshotTypeVisitor(TypeVisitor[SnapshotItem]):
         required = tuple(sorted(typ.required_keys))
         return ('TypedDictType', items, required)
 
+    def visit_literal_type(self, typ: LiteralType) -> SnapshotItem:
+        return ('LiteralType', typ.value, snapshot_type(typ.fallback))
+
     def visit_union_type(self, typ: UnionType) -> SnapshotItem:
         # Sort and remove duplicates so that we can use equality to test for
         # equivalent union type snapshots.
