@@ -38,13 +38,13 @@ Notes:
 """
 
 import contextlib
-from typing import Union, Iterator, Optional
+from typing import Union, Iterator, Optional, List
 
 from mypy.nodes import (
     Node, FuncDef, NameExpr, MemberExpr, RefExpr, MypyFile, FuncItem, ClassDef, AssignmentStmt,
     ImportFrom, Import, TypeInfo, SymbolTable, Var, CallExpr, Decorator, OverloadedFuncDef,
     SuperExpr, UNBOUND_IMPORTED, GDEF, MDEF, IndexExpr, SymbolTableNode, ImportAll, TupleExpr,
-    ListExpr, ForStmt, Block
+    ListExpr, ForStmt, Block, SymbolNode,
 )
 from mypy.semanal_shared import create_indirect_imported_name
 from mypy.traverser import TraverserVisitor
@@ -81,6 +81,7 @@ class NodeStripVisitor(TraverserVisitor):
         self.names = file_node.names
         self.file_node = file_node
         self.recurse_into_functions = False
+        file_node.plugin_deps.clear()
         file_node.accept(self)
 
     def visit_block(self, b: Block) -> None:
@@ -90,14 +91,17 @@ class NodeStripVisitor(TraverserVisitor):
 
     def visit_class_def(self, node: ClassDef) -> None:
         """Strip class body and type info, but don't strip methods."""
-        self.strip_type_info(node.info)
+        to_delete = set(self.strip_type_info(node.info))
         node.type_vars = []
         node.base_type_exprs.extend(node.removed_base_type_exprs)
         node.removed_base_type_exprs = []
+        node.defs.body = [s for s in node.defs.body
+                          if s not in to_delete]
+
         with self.enter_class(node.info):
             super().visit_class_def(node)
 
-    def strip_type_info(self, info: TypeInfo) -> None:
+    def strip_type_info(self, info: TypeInfo) -> List[SymbolNode]:
         info.type_vars = []
         info.bases = []
         info.is_abstract = False
@@ -111,6 +115,13 @@ class NodeStripVisitor(TraverserVisitor):
         info.declared_metaclass = None
         info.metaclass_type = None
 
+        # We need to delete any entries that were generated by plugins,
+        # since they will get regenerated.
+        to_delete = [(k, v) for k, v in info.names.items() if v.plugin_generated]
+        for k, _ in to_delete:
+            del info.names[k]
+        return [v.node for k, v in to_delete if v.node]
+
     def visit_func_def(self, node: FuncDef) -> None:
         if not self.recurse_into_functions:
             return
