@@ -5,7 +5,7 @@ from mypy.types import (
     Type, AnyType, UnboundType, TypeVisitor, FormalArgument, NoneTyp, function_type,
     Instance, TypeVarType, CallableType, TupleType, TypedDictType, UnionType, Overloaded,
     ErasedType, PartialType, DeletedType, UninhabitedType, TypeType, is_named_instance,
-    FunctionLike, TypeOfAny
+    FunctionLike, TypeOfAny, LiteralType,
 )
 import mypy.applytype
 import mypy.constraints
@@ -24,11 +24,15 @@ from mypy.typestate import TypeState, SubtypeKind
 
 from mypy import experiments
 
+MYPY = False
+if MYPY:
+    from typing_extensions import Final
+
 
 # Flags for detected protocol members
-IS_SETTABLE = 1
-IS_CLASSVAR = 2
-IS_CLASS_OR_STATIC = 3
+IS_SETTABLE = 1  # type: Final
+IS_CLASSVAR = 2  # type: Final
+IS_CLASS_OR_STATIC = 3  # type: Final
 
 
 TypeParameterChecker = Callable[[Type, Type, int], bool]
@@ -323,6 +327,9 @@ class SubtypeVisitor(TypeVisitor[bool]):
         else:
             return False
 
+    def visit_literal_type(self, t: LiteralType) -> bool:
+        raise NotImplementedError()
+
     def visit_overloaded(self, left: Overloaded) -> bool:
         right = self.right
         if isinstance(right, Instance):
@@ -774,10 +781,10 @@ def is_callable_compatible(left: CallableType, right: CallableType,
     if right.is_ellipsis_args:
         return True
 
-    left_star = left.var_arg
-    left_star2 = left.kw_arg
-    right_star = right.var_arg
-    right_star2 = right.kw_arg
+    left_star = left.var_arg()
+    left_star2 = left.kw_arg()
+    right_star = right.var_arg()
+    right_star2 = right.kw_arg()
 
     # Match up corresponding arguments and check them for compatibility. In
     # every pair (argL, argR) of corresponding arguments from L and R, argL must
@@ -1164,6 +1171,9 @@ class ProperSubtypeVisitor(TypeVisitor[bool]):
             return True
         return self._is_proper_subtype(left.fallback, right)
 
+    def visit_literal_type(self, left: LiteralType) -> bool:
+        raise NotImplementedError()
+
     def visit_overloaded(self, left: Overloaded) -> bool:
         # TODO: What's the right thing to do here?
         return False
