@@ -27,9 +27,26 @@ T = TypeVar('T')
 
 JsonDict = Dict[str, Any]
 
-# If we import type_visitor in the middle of the file, mypy breaks, and if we do it
-# at the top, it breaks at runtime because of import cycle issues, so we do it at different
-# times in different places.
+# The set of all valid expressions that can currently be contained
+# inside of a Literal[...].
+#
+# Literals can contain enum-values: we special-case those and
+# store the value as a string.
+#
+# TODO: confirm that we're happy with representing enums (and the
+# other types) in the manner described above.
+#
+# Note: this type also happens to correspond to types that can be
+# directly converted into JSON. The serialize/deserialize methods
+# of 'LiteralType' rely on this, as well 'server.astdiff.SnapshotTypeVisitor'
+# and 'types.TypeStrVisitor'. If we end up adding any non-JSON-serializable
+# types to this list, we should make sure to edit those methods to match.
+LiteralValue = Union[int, str, bool, None]
+
+# If we only import type_visitor in the middle of the file, mypy
+# breaks, and if we do it at the top, it breaks at runtime because of
+# import cycle issues, so we do it at the top while typechecking and
+# then again in the middle at runtime.
 if MYPY:
     from mypy.type_visitor import TypeVisitor, SyntheticTypeVisitor, TypeTranslator, TypeQuery
 
@@ -1239,6 +1256,54 @@ class TypedDictType(Type):
             yield (item_name, None, right_item_type)
 
 
+class LiteralType(Type):
+    """The type of a Literal instance. Literal[Value]
+
+    A Literal always consists of:
+
+    1. A native Python object corresponding to the contained inner value
+    2. A fallback for this Literal. The fallback also corresponds to the
+       parent type this Literal subtypes.
+
+    For example, 'Literal[42]' is represented as
+    'LiteralType(value=42, fallback=instance_of_int)'
+    """
+    __slots__ = ('value', 'fallback')
+
+    def __init__(self, value: LiteralValue, fallback: Instance,
+                 line: int = -1, column: int = -1) -> None:
+        super().__init__(line, column)
+        self.value = value
+        self.fallback = fallback
+
+    def accept(self, visitor: 'TypeVisitor[T]') -> T:
+        return visitor.visit_literal_type(self)
+
+    def __hash__(self) -> int:
+        return hash((self.value, self.fallback))
+
+    def __eq__(self, other: object) -> bool:
+        if isinstance(other, LiteralType):
+            return self.fallback == other.fallback and self.value == other.value
+        else:
+            return NotImplemented
+
+    def serialize(self) -> Union[JsonDict, str]:
+        return {
+            '.class': 'LiteralType',
+            'value': self.value,
+            'fallback': self.fallback.serialize(),
+        }
+
+    @classmethod
+    def deserialize(cls, data: JsonDict) -> 'LiteralType':
+        assert data['.class'] == 'LiteralType'
+        return LiteralType(
+            value=data['value'],
+            fallback=Instance.deserialize(data['fallback']),
+        )
+
+
 class StarType(Type):
     """The star type *type_parameter.
 
@@ -1542,10 +1607,9 @@ class ForwardRef(Type):
 # to make it easier to gradually get modules working with mypyc.
 # Import them here, after the types are defined.
 # This is intended as a re-export also.
-if not MYPY:
-    from mypy.type_visitor import (  # noqa
-        TypeVisitor, SyntheticTypeVisitor, TypeTranslator, TypeQuery
-    )
+from mypy.type_visitor import (  # noqa
+    TypeVisitor, SyntheticTypeVisitor, TypeTranslator, TypeQuery
+)
 
 
 class TypeStrVisitor(SyntheticTypeVisitor[str]):
@@ -1693,6 +1757,9 @@ class TypeStrVisitor(SyntheticTypeVisitor[str]):
                 suffix = ', fallback={}'.format(t.fallback.accept(self))
         return 'TypedDict({}{}{})'.format(prefix, s, suffix)
 
+    def visit_literal_type(self, t: LiteralType) -> str:
+        return 'Literal[{}]'.format(repr(t.value))
+
     def visit_star_type(self, t: StarType) -> str:
         s = t.type.accept(self)
         return '*{}'.format(s)
